#-------------------------------------------------------------------#
#  Package:  EMMIX-WIRE (including extensions) 
#  Version:  1.0-1
#  Depends:  R (>=2.0)
#  Authors:  Angus Ng, Geoff McLachlan, Kui Wang
#  UpDate   :  22 Jan, 2008.
#  Maintainer: Kui Wang
#  Copyright: No distribution without permission.
#  
#  Changes:
#  1. On March 12, 2007, a dynamic link library (DLL) 'emmixwire.dll' 
#     is created based on Angus Ng's Fortran program 'emmix-wire.f'
#  2. On March 20, 2007, some functions are removed to make this package 
#     not rely on mvtnorm 
#  3. On June 18, Tao is added.
#  
#  4. On 22 August, 2007. option of covariance structure of gene effects: ncov =1 (common covariane), 2 (common idagonal), 
#      3 (genreal) , 4 (generasl diagonal) , 5 ( thetau(k)* I, k=1,...,ng), 0 (turn off)
#  5. Option of cluster effect can be switch off if ncov=0.

#-------------------------------------------------------------------#


# Beginning of the Program


#----------------------------------------------------------
# Part III: subroutinres for initial values
# initial values for regression based models
#----------------------------------------------------------
#density of multivariate normal distribution
dmvn<-function(y,mean = rep(0, nrow(sigma)), sigma = diag(length(mean)),log=FALSE)
{
y<-as.matrix(y)
sigma<-as.matrix(sigma)
if((det=det(sigma))==0) stop("singular matrix")
if((p=ncol(sigma))>1 && (ncol(y)==1)) y<-t(y)
inv<-solve(sigma)
fdist<-function(x) c(t(x-mean)%*%inv%*%(x-mean))
dist<-apply(y,FUN=fdist,1)

lk<--0.5*(dist+log(2*pi)*p+log(det))
ifelse(dist>375,0,exp(lk))
}

init.fit<-function(dat,xsxc,qe,m,n,ng,nos)
{

found<-NULL
found$loglik<--Inf

rand<-seq(from=100,by=100,length=10)

for(j in 1:length(rand))
{
set.seed(rand[j])

initobj<-init.km(dat,xsxc,qe,m,n,ng,nos)
if(length(initobj)==6)
if(initobj$loglik>found$loglik)
found<-initobj

} #end of loop

found
}

init.km<-function(dat,xsxc,qe,m,n,ng,nos)
{
#
cluster<-rep(1,n)

if(ng>1)
cluster<- kmeans(dat,ng,nstart=5)$cluster
init.reg(xsxc,qe,cluster,m,n,ng,dat)
}

# initial values for given clustering
init.reg<-function(xsxc,qe,clust,m,n,K,dat)
{
# set x for regression
x<-as.matrix(xsxc)
beta<-matrix(0,nrow=ncol(x),ncol=K)

sigma<-pai<-rep(0,K)
#
for( ij in 1:K)
{
ni<-sum(clust==ij)
y<-as.vector(t(dat[clust==ij,]))

if(ni>1)
{
for(i in 2:ni)
x<-rbind(x,xsxc)
}

#dimnames(x)<-list(NULL,paste("x",1:ncol(x),sep=''))
obj<-lm(y~0+x)

beta[,ij]<-coef(obj)
sigma[ij]<-sum((resid(obj))^2)/length(y)
pai[ij]<-ni/n

x<-as.matrix(xsxc) # reset x for next iteration

}
# loglikelihood

fhi<-matrix(0,ncol=K,nrow=n)

for(h in 1:K)
	fhi[,h]<-dmvn(dat,c(xsxc%*%beta[,h]),diag(sigma[h],m))


tao<-1 #t(t(fhi)*pai)/rowSums(t( t(fhi)*pai))

loglik<-sum(log(colSums(t(fhi)*pai)))

thete<-matrix(0,ncol=K,nrow=qe)
for(i in 1:qe)
thete[i,]<-sigma

list(beta=beta,thete=thete,pai=pai,loglik=loglik,tao=tao,dfhij=1)

}

rand.index<-function (x, y)
{
     x <- as.vector(x)
     y <- as.vector(y)
     xx <- outer(x, x, "==")
     yy <- outer(y, y, "==")
     upper <- row(xx) < col(xx)
     xx <- xx[upper]
     yy <- yy[upper]
     a <- sum(as.numeric(xx & yy))
     b <- sum(as.numeric(xx & !yy))
     c <- sum(as.numeric(!xx & yy))
     d <- sum(as.numeric(!xx & !yy))
     ni <- (b + a)
     nj <- (c + a)
     abcd <- a + b + c + d
     q <- (ni * nj)/abcd
#  rand index
     rd1<-(a+d)/abcd
#  adjusted rand index
     rda<-(a - q)/((ni + nj)/2 - q)
c(rd1,rda)

}



#---------------------
fit.emmix.wire<-function(dat,X,W,U,V,pai,beta,thete,n,m,grp,ncov,nvcov,itmax,epsilon)
{
if(!is.loaded('emmixwire.dll'))
dyn.load('emmixwire.dll')
#
# Assuming the X,U,V,W are properly specified!!!

ret<-.Fortran('emwire',
as.double(dat),
as.integer(n),
as.integer(m),
qb<-as.integer(ncol(X)),
s1<-as.integer(ncol(U)),
s2<-as.integer(ncol(V)),
qe<-as.integer(ncol(W)),
g<-as.integer(grp),
as.double(X),
as.double(U),
as.double(V),
as.double(W),  # item 12
as.integer(ncov),
as.integer(nvcov),
as.integer(itmax),#item 15
as.double(epsilon),
pai =as.double(pai),
beta=as.double(beta), # beta(qb,ng)
sigma=as.double(thete),# thete(qe,ng)
thetu=double(g*s1*s1),#
thetv=double(g),
clust=integer(n),
loglik=double(1),
BIC=double(1),
AIC=double(1),
ll=double(itmax),
tau=double(n*grp),
ierr=integer(1))[17:28] 
# 23 items


#dyn.unload('emmixwire.dll')

list(pai=ret$pai,
beta  = array(ret$beta, c(qb,g)),
sigma = array(ret$sigma,c(qe,g)),
thetau= array(ret$thetu,c(s1,s1,g)),
thetav= ret$thetv,
loglik=ret$loglik,
BIC=ret$BIC,
#AIC=ret$AIC,
error=ret$ierr,
#lk=ret$ll,
clust=  ret$clust)
#tao=array(ret$tau,c(n,g)))
}

#-----------------------------------#


emmixwire<-function(dat,grp,X,W,Z1,Z2,debug=0,ncov=3,nvcov=0,itmax=1000,epsilon=0.00001)
{
#----------------
X<-as.matrix(X)
W<-as.matrix(W)
U<-as.matrix(Z1)
V<-diag(20)

if(!is.null(Z2)& nvcov==1)
V<-as.matrix(Z2)

#----------------

dat<-as.matrix(dat)
n<-nrow(dat)
m<-ncol(dat)
qe<-ncol(W)

found<-init.fit(dat,X,qe,m,n,grp,nos=2)
#
fit.emmix.wire(dat,X,W,U,V,found$pai,found$beta,found$thete,n,m,grp,ncov,nvcov,itmax,epsilon)

}

# End of program

#-----------------------------------------
#  example:  simulated data
#-----------------------------------------


#  step 1:  figure out the X,Z1,Z2,W

#m<-20   # m is number of columns

#Z1<-W<-rep(1,m); 
#Z2<-diag(m)

# the data is simulated as a time series as follows

#time<-seq(from=0,by=10,length=m)
#X<-cbind(cos(2*pi*time/100),sin(2*pi*time/100))

# step 2: extract the data from local file
# a normal way is using 'dat<-read.table('datfile'), or read.cvs('datfile')
# if read from a text file

#datobj<-dget('tmdat1.txt')
#dat<-as.matrix(datobj$dat)


# step 3: specify the number of clusters you'll try
#grp<-4

# step 4: call the main function to fit emmix-wire
#obj<-emmixwire(dat,X,Z1,Z2,W,grp)

# step 5: print out the results for analysis

#print(obj$clust)

# step 6: others,   
# for example: calculate the rand index
 
#rand.index(datobj$clust,obj$clust)

